#
# ExcoLight Color Python Control
#
# No classes, no tricks, just a quick and dirty, tiny demonstration.
# 
#
# NOTE: CODE USES TABs (best viewed with TAB==2).
#
#
# requires:
#
#   - Python
#     http://www.python.org
#
#   - pySerial
#     http://pyserial.sourceforge.net/
#
#
# (c)2010, ASkr; www.askrprojects.net
#




# SET YOUR COM PORT HERE (1=COM1, 2=COM2, ...)
COMPORT = 17

# NUMBER OF EXCOLIGHT COLOR MODULES
MODULES = 1




import serial
import random
import sys
import time


# some fishy "defines ;-)"
CMD_RESET       = 0x00
CMD_SET_ADDRESS = 0x01
CMD_SET_PWM     = 0x02
CMD_SET_123     = 0x03
CMD_SET_RGB     = 0x04
CMD_GET_REG     = 0x05
CMD_RAW_DATA    = 0x06
CMD_SEND_ECMD   = 0x07


try:
	sOut = serial.Serial(COMPORT-1, 4800, timeout=0, parity=serial.PARITY_ODD)
except:
	print("unable to open serial port "+str(COMPORT))
	exit(-1)
	
	

##################################################################################################
### Send
###
### Send a packet on the bus.
###
### WARNING: no error/type checks!
def Send(address, command, dbyte1, dbyte2, dbyte3):
	address&=0x7f
	address|=0x80
	command&=0x07
	dbyte1&=0xff
	dbyte2&=0xff
	dbyte3&=0xff
	if dbyte1 & 0x80:
		command|=0x20
		dbyte1&=0x7f
	if dbyte2 & 0x80:
		command|=0x10
		dbyte2&=0x7f
	if dbyte3 & 0x80:
		command|=0x08
		dbyte3&=0x7f
	i=sOut.write(chr(address))
	i+=sOut.write(chr(command))
	i+sOut.write(chr(dbyte1))
	i+sOut.write(chr(dbyte2))
	i+sOut.write(chr(dbyte3))
	return i

	

##################################################################################################
### EL_SetAddress
###
### Sets a new address <nadr> for module <adr>
###
### WARNING: no error/type checks!
def EL_SetAddress(adr, nadr, gadr):
	Send(adr,CMD_SET_ADDRESS,nadr,gadr,0x55)

	
	
##################################################################################################
### EL_Reset
###
### Resets module(s) with address <adr>
###
### WARNING: no error/type checks!
def EL_Reset(adr):
	Send(adr,CMD_RESET,0xaa,0x55,0)



##################################################################################################
### EL_Set123
###
### Sets channels 1, 2 and 3 on module <adr> to values <ch1>, <ch2>, <ch3>
###
### WARNING: no error/type checks!
def EL_Set123(adr, ch1, ch2, ch3):
	Send(adr,CMD_SET_123,ch1,ch2,ch3)

	
	
##################################################################################################
### EL_SetRGB
###
### Sets channels R, G and B on module <adr> to values <vr>, <vg>, <vb>
###
### WARNING: no error/type checks!
def EL_SetRGB(adr, vr, vg, vb):
	Send(adr,CMD_SET_RGB,vr,vg,vb)


	
	
	
	
##################################################################################################
### main
###
if __name__=="__main__":

	# power up the bus
	for i in range(MODULES):
		EL_SetAddress(0,i+1,0)
		time.sleep(1)

	# ramp white
	for n in range(0,100,5):
		EL_Set123(0,n,0,0)
		time.sleep(0.04)
	for n in range(100,-1,-5):
		EL_Set123(0,n,0,0)
		time.sleep(0.04)
	
	# ramp red
	for n in range(0,100,5):
		EL_SetRGB(0,n,0,0)
		time.sleep(0.04)
	for n in range(100,-1,-5):
		EL_SetRGB(0,n,0,0)
		time.sleep(0.04)
	
	# ramp green
	for n in range(0,100,5):
		EL_SetRGB(0,0,n,0)
		time.sleep(0.04)
	for n in range(100,-1,-5):
		EL_SetRGB(0,0,n,0)
		time.sleep(0.04)

	# ramp blue
	for n in range(0,100,5):
		EL_SetRGB(0,0,0,n)
		time.sleep(0.04)
	for n in range(100,-1,-5):
		EL_SetRGB(0,0,0,n)
		time.sleep(0.04)
		
	# quick change, full power
	for i in range(6):
		EL_SetRGB(0,255,0,0)
		time.sleep(0.3)
		EL_SetRGB(0,0,255,0)
		time.sleep(0.3)
		EL_SetRGB(0,0,0,255)
		time.sleep(0.3)
	
	EL_SetRGB(0,0,0,0)
	time.sleep(0.5)

	# pseudo rainbow ;)
	for n in range(40):
		EL_SetRGB(0,n,0,0)
		time.sleep(0.04)
	for n in range(40):
		EL_SetRGB(0,40,n,0)
		time.sleep(0.04)
	for n in range(40,-1,-1):
		EL_SetRGB(0,n,40,0)
		time.sleep(0.04)
	for n in range(40):
		EL_SetRGB(0,0,40,n)
		time.sleep(0.04)
	for n in range(40,-1,-1):
		EL_SetRGB(0,0,n,40)
		time.sleep(0.04)
	for n in range(40,-1,-1):
		EL_SetRGB(0,0,0,n)
		time.sleep(0.04)


	# reset bus
	EL_Reset(0)

		
